package cn.com.infosec.asn1;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

public abstract class ASN1Set extends ASN1Object {
   protected Vector set;

   public static ASN1Set getInstance(Object obj) {
      if (obj != null && !(obj instanceof ASN1Set)) {
         throw new IllegalArgumentException("unknown object in getInstance: " + obj.getClass().getName());
      } else {
         return (ASN1Set)obj;
      }
   }

   public static ASN1Set getInstance(ASN1TaggedObject obj, boolean explicit) {
      if (explicit) {
         if (obj.isExplicit()) {
            return (ASN1Set)obj.getObject();
         } else {
            throw new IllegalArgumentException("object implicit - explicit expected.");
         }
      } else if (obj.isExplicit()) {
         return new DERSet(obj.getObject());
      } else if (obj.getObject() instanceof ASN1Set) {
         return (ASN1Set)obj.getObject();
      } else {
         ASN1EncodableVector explicit1;
         explicit1 = new ASN1EncodableVector.<init>();
         if (!(obj.getObject() instanceof ASN1Sequence)) {
            throw new IllegalArgumentException("unknown object in getInstance: " + obj.getClass().getName());
         } else {
            Enumeration var2 = ((ASN1Sequence)obj.getObject()).getObjects();

            while(var2.hasMoreElements()) {
               explicit1.add((DEREncodable)var2.nextElement());
            }

            return new DERSet(explicit1, false);
         }
      }
   }

   public ASN1Set() {
      ASN1Set var10000 = this;
      super();
      Vector var1;
      var1 = new Vector.<init>();
      var10000.set = var1;
   }

   private boolean lessThanOrEqual(byte[] a, byte[] b) {
      int var3;
      int var4;
      int var5;
      if (a.length <= b.length) {
         for(var5 = 0; var5 != a.length; ++var5) {
            var3 = a[var5] & 255;
            if ((var4 = b[var5] & 255) > var3) {
               return true;
            }

            if (var3 > var4) {
               return false;
            }
         }

         return true;
      } else {
         for(var5 = 0; var5 != b.length; ++var5) {
            var3 = a[var5] & 255;
            if ((var4 = b[var5] & 255) > var3) {
               return true;
            }

            if (var3 > var4) {
               return false;
            }
         }

         return false;
      }
   }

   private byte[] getEncoded(DEREncodable obj) {
      ByteArrayOutputStream var3;
      ByteArrayOutputStream var10000 = var3 = new ByteArrayOutputStream;
      var10000.<init>();
      DEREncodable var10002 = obj;
      ASN1OutputStream var10001 = new ASN1OutputStream(var3);

      try {
         var10001.writeObject(var10002);
      } catch (IOException var2) {
         throw new IllegalArgumentException("cannot encode object added to SET");
      }

      return var10000.toByteArray();
   }

   public Enumeration getObjects() {
      return this.set.elements();
   }

   public DEREncodable getObjectAt(int index) {
      return (DEREncodable)this.set.elementAt(index);
   }

   public int size() {
      return this.set.size();
   }

   public ASN1SetParser parser() {
      return new ASN1SetParser() {
         private final int max = ASN1Set.this.size();
         private int index;

         public DEREncodable readObject() {
            int var1;
            if ((var1 = this.index) == this.max) {
               return null;
            } else {
               ASN1Set var10000 = ASN1Set.this;
               this.index = var1 + 1;
               DEREncodable var2;
               if ((var2 = var10000.getObjectAt(var1)) instanceof ASN1Sequence) {
                  return ((ASN1Sequence)var2).parser();
               } else {
                  return (DEREncodable)(var2 instanceof ASN1Set ? ((ASN1Set)var2).parser() : var2);
               }
            }
         }

         public DERObject getDERObject() {
            return ASN1Set.this;
         }
      };
   }

   public int hashCode() {
      Enumeration var2 = this.getObjects();

      int var1;
      for(var1 = 0; var2.hasMoreElements(); var1 ^= var2.nextElement().hashCode()) {
      }

      return var1;
   }

   boolean asn1Equals(DERObject o) {
      if (!(o instanceof ASN1Set)) {
         return false;
      } else {
         ASN1Set o1 = (ASN1Set)o;
         if (this.size() != o1.size()) {
            return false;
         } else {
            Enumeration var4 = this.getObjects();
            Enumeration o2 = o1.getObjects();

            DERObject var2;
            DERObject var3;
            do {
               do {
                  if (!var4.hasMoreElements()) {
                     return true;
                  }
               } while((var2 = ((DEREncodable)var4.nextElement()).getDERObject()) == (var3 = ((DEREncodable)o2.nextElement()).getDERObject()));
            } while(var2 != null && var2.equals(var3));

            return false;
         }
      }
   }

   protected void sort() {
      if (this.set.size() > 1) {
         boolean var1 = true;

         boolean var5;
         for(int var2 = this.set.size() - 1; var1; var1 = var5) {
            int var8 = 0;
            int var3 = 0;
            byte[] var4 = this.getEncoded((DEREncodable)this.set.elementAt(0));

            int var6;
            for(var5 = false; var8 != var2; var8 = var6) {
               byte[] var7;
               if (!this.lessThanOrEqual(var4, var7 = this.getEncoded((DEREncodable)this.set.elementAt(var6 = var8 + 1)))) {
                  Object var9 = this.set.elementAt(var8);
                  Vector var10001 = this.set;
                  var10001.setElementAt(var10001.elementAt(var6), var8);
                  this.set.setElementAt(var9, var6);
                  var5 = true;
                  var7 = var4;
                  var3 = var8;
               }

               var4 = var7;
            }

            var2 = var3;
         }
      }

   }

   protected void addObject(DEREncodable obj) {
      this.set.addElement(obj);
   }

   abstract void encode(DEROutputStream var1);

   public String toString() {
      return this.set.toString();
   }
}
